/*
    SaveStateCmd.cc

    Will Eighteen: will@opaline.demon.co.uk
*/

#include "wave/waveclass.h"
#include "AppWindow.h"
#include "SaveStateCmd.h"
#include "OpResources.h"
#include <InfoDialogManager.h>
#include <MessageDialogManager.h>
#include <FileAccess.h>
#include <FileSelect.h>


SaveStateCmd::SaveStateCmd ( char *name, int active ) : NoUndoCmd ( name, active )
{
	_name = name;
	_popupRealised = False;
}


void SaveStateCmd::doCommand()
{
	if ( _popupRealised == True )
		return;

	char title[BUFSIZE];
	
	snprintf ( title, sizeof ( title ), "%s file", _name );
	title [ sizeof ( title ) - 1 ] = 0;
	_fileSelect = new FileSelect ( ( char * ) title, TRUE,
								&SaveStateCmd::stateSelectedCallback, NULL,
								&SaveStateCmd::stateCancelledCallback, ( void * ) this );

	_fileSelect->execute();
	_popupRealised = True;
}


void SaveStateCmd::stateSelectedCallback ( void *clientData, char *filename )
{
	SaveStateCmd *obj = ( SaveStateCmd * ) clientData;

	if ( filename[strlen(filename) - 1] != '/' )	// no file specified
		obj->stateSelected ( filename );

	obj->_popupRealised = False;
	delete obj->_fileSelect;
}


void SaveStateCmd::stateSelected ( char *filename )
{
// put 'save state on exit' toggle in options?
	char buf[BUFSIZE];
	int error;

	FileAccess *chk = new FileAccess ( filename, "r" );
	error = chk->errNum();
	delete chk;
	
	_filename = filename;
	
	if ( error == 0 ) 	// file exists
	{
		if ( mixwaveResources->enableWarningPopups() == True )
		{
			snprintf ( buf, sizeof ( buf ), "%s\nFile exists - overwrite?", filename );
			buf [ sizeof ( buf ) - 1 ] = 0;
			
			infoDialogManager->post ( buf, ( void * ) this,
										&SaveStateCmd::saveStateOkCallback,
										&SaveStateCmd::stateCancelledCallback );
		}
	}
	else
		saveState();
}


void SaveStateCmd::stateCancelledCallback ( void *clientData )
{
	SaveStateCmd *obj = ( SaveStateCmd * ) clientData;

	obj->_popupRealised = False;
	delete obj->_fileSelect;
}


void SaveStateCmd::saveStateOkCallback ( void *clientData )
{
	SaveStateCmd *obj = ( SaveStateCmd * ) clientData;

	obj->saveState();
}


void SaveStateCmd::saveState()
{
	if ( ( wavePak->Count() == 0 ) && ( templatePak->Count() == 0 ) )	// nothing to save
	{
		char msg[48];
		strcpy ( msg, "Save state failed - no waves/templates loaded." );
		messageDialogManager->post ( msg );
		return;
	}

	char buf[BUFSIZE];

	FileAccess *out = new FileAccess ( _filename, "w" );
	
	if ( out->errNum() != 0 )
	{
		snprintf ( buf, sizeof ( buf ), "%s\nError: %s", _filename, out->error() );
		buf [ sizeof ( buf ) - 1 ] = 0;
		messageDialogManager->post ( buf );
		delete out;
		return;
	}
	
	int count = 0;
	int numWaves = wavePak->Count();
	int numTemplates = templatePak->Count();
	int nameLength;
	
	out->numElements ( 1 );
	out->elementSize ( sizeof ( int ) );

	out->writeF ( ( void * ) &numWaves );
	out->writeF ( ( void * ) &numTemplates );

	if ( wavePak->Count() > 0 )
	{
		for ( count = 0; count < wavePak->Count(); count++ )
		{
			nameLength = strlen ( (*wavePak)[count].pWave->name() );
			out->elementSize ( sizeof ( int ) );
			out->writeF ( ( void * ) &nameLength );
			out->elementSize ( nameLength );
			out->writeF ( (*wavePak)[count].pWave->name() );
			out->elementSize ( sizeof ( *wavePak ) );
			out->writeF ( &(*wavePak)[count] );
		}
	}
	
	if ( templatePak->Count() > 0 )
	{
		for ( count = 0; count < templatePak->Count(); count++ )
		{
			nameLength = strlen ( (*templatePak)[count].name );
			out->elementSize ( sizeof ( int ) );
			out->writeF ( ( void * ) &nameLength );
			out->elementSize ( nameLength );
			out->writeF ( (*templatePak)[count].name );
			nameLength = strlen ( (*templatePak)[count].syncFile );
			out->elementSize ( sizeof ( int ) );
			out->writeF ( ( void * ) &nameLength );
			out->elementSize ( nameLength );
			out->writeF ( (*templatePak)[count].syncFile );
			out->elementSize ( sizeof ( *templatePak ) );
			out->writeF ( &(*templatePak)[count] );
		}
	}
	
	delete out;
	
	// post nice comforting ok message
	if ( mixwaveResources->enableMessagePopups() == True )
	{
		snprintf ( buf, sizeof ( buf ), "%s\nState file saved.", _filename );
		buf [ sizeof ( buf ) - 1 ] = 0;
		messageDialogManager->post ( buf );	
	}
}
