/* DMenuCmd.h */

#ifndef DMENUCMD_H
#define DMENUCMD_H

#define MENUSTORAGESPACE 4096

#include "NoUndoCmd.h"
#include <Xm/Xm.h>


typedef void ( *DMenuCmdCallback ) ( void *, unsigned int, char * );


class DMenuCmd;
struct menuWidgetSet { int n; DMenuCmd *menu; };


class DMenuCmd : public NoUndoCmd
{
	public:
		// name, parent widget, menu popup label, 
		// menu activate button label, menu entry list title,
		// menu entries
		DMenuCmd ( const char *, int active, Boolean quitOnSelect,
					DMenuCmdCallback, void *, char *, ... );
					
		// instead of fixed string menu entries, will also accept
		// no. of items in menu, size of char array storing entries,
		// pointer to base address of char array
		DMenuCmd ( const char *, int active, Boolean quitOnSelect,
					 DMenuCmdCallback, void *, char *, int, int, char * );
		
		~DMenuCmd();
		virtual const char *const className() { return ( "DMenuCmd" ); }
		void setTitle ( char * );
		
	protected:
		void doCommand();		// called by base class
		DMenuCmdCallback _callback;	// function to be called
		void *_clientData;		// data provided by caller

	private:
		static void menuEntrySelectCallback( Widget, XtPointer, XtPointer );

		// callback for WM 'X'
		static void clientMessageProc ( Widget, XtPointer, XtPointer );

 		void menuEntrySelect ( unsigned int );
		void setSelectedItem ( int n ) { _selectedItemNumber = n; }
		void quit();

		Boolean _fixedString;
		Boolean _quitOnSelect;	// if menu is to be popped down on selection
		Boolean _popupRealised;
		Widget _popup, _menuLabel, _menu, _entry;
		char _name[128];
		char _menuTitle[128];
		char _applicationName[128];
		char *_pItemArray;
		char _menuItemList[MENUSTORAGESPACE];
		char *_menuLabelText;
		unsigned int _numMenuItems;
		int _charArraySize;
		int _selectedItemNumber;
		menuWidgetSet ** arrayOfPointersToMemBlocks;
};

#endif

/*
  Notes:
  
  Since we can't use varargs ploy to access the menu item list outside the
  constructor, in doCommand, we must store the data internally. Thus the menu
  item capability is restricted to this storage limit, 'MENUSTORAGESPACE'.
  
  Usage:
  
  Fixed string version 

  Cmd *dMenu = new DMenuCmd ( "DMenu", TRUE, &AppWindow::dMenuCallback,
								( void * ) this, "Dinner",
								"Fish","Chips","Quit",NULL );

  Pointer to char array version
  
  int numMenuItems = 4;
  int lengthOfCharArray = 10;
  char strings [ numMenuItems ] [ lengthOfCharArray ];

  strcpy ( strings[0], "Plum" );
  strcpy ( strings[1], "Peach" );
  strcpy ( strings[2], "Pear" );
  strcpy ( strings[3], "Quit" );

  Cmd *dMenu = new DMenuCmd ( "DMenu", TRUE, &AppWindow::dMenuCallback,
								( void * ) this, "Access Fruit!",
								numMenuItems, lengthOfCharArray, ( char * )
								&strings );

  Callback definition and implementation
	
  static void dMenuCallback ( void *, int );

  void AppWindow::dMenuCallback ( void *clientData, int selectedItemNum )
  {
    printf("item number selected %d\n", selectedItemNum);
  }

  Note that each menu must reserve the last menu item for closing the popup
  menu.
  
  If quitOnSelect is true, the menu will be popped down when a selection has
  been made; in this case, the menu item number returned will always be valid
  with reference to the labels with which it was supplied on construction.
  
  This is not the case where quitOnSelect is false; here the menu remains in
  existence once popped up until 'quit' is selected. However, it is possible that
  the underlying strings with which the menu was constructed may have changed
  whilst the menu is in existence. For this reason, the callback also returns a
  pointer to the label string corresponding to the menu item number; this can
  be used to check whether the reference returned on callback is valid (i.e. to
  a currently existing string).
  
  The setTitle method enables the menu title label to be set externally if this
  cannot be determined at the time of object instantiation.
*/
