/*
  LineEdit.cc
 
  Will Eighteen, 2000: will@opaline.demon.co.uk             

  This program is free software; you can redistribute it and/or modify       
  it under the terms of the GNU General Public License as published by       
  the Free Software Foundation; either version 2 of the License, or          
  (at your option) any later version.                                        

  This program is distributed in the hope that it will be useful,            
  but WITHOUT ANY WARRANTY; without even the implied warranty of             
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              
  GNU General Public License for more details.                               

  You should have received a copy of the GNU General Public License          
  along with this program; if not, write to the Free Software                
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                  

  See COPYING for full details.                                  
*/

#include "LineEdit.h"
#include "Application.h"
#include <string.h>
#include <Xm/Label.h>
#include <Xm/TextF.h>
#include <Xm/RowColumn.h>
#include <Xm/PushB.h>
#include <Xm/Protocols.h>
#include <stdio.h>


LineEdit::LineEdit ( char *name, int active, TextLineCallback callback,
		    TextLineCancelledCallback cancelCallback, void *clientData ) : NoUndoCmd ( name, active )
{
	_callback = callback;
	_cancelCallback = cancelCallback;
	_clientData = clientData;
	_helpText[0] = 0;
	_presetText = 0;
	strncpy ( _applicationName, ( char * ) application->applicationClass(), sizeof ( _applicationName ) );
	_applicationName [ sizeof ( _applicationName ) - 1 ] = 0;
	strncat ( _applicationName, " Text Editor", sizeof (_applicationName ) - strlen ( _applicationName ) - 1 );
	// or the popup inherits the size of the top-level window
	strncpy ( _name, name, sizeof ( _name ) );
	_name [ sizeof ( _name ) -1 ] = 0;
}


void LineEdit::doCommand()
{
	int n;
	Arg args[4];
	Atom WM_DELETE_WINDOW;
	
	_popup = XtCreatePopupShell ( _applicationName, transientShellWidgetClass,
									application->baseWidget(), NULL, 0 );
									
	_rc1 = XtVaCreateManagedWidget ( "RowColumn1", xmRowColumnWidgetClass, _popup,
				XmNorientation, XmVERTICAL,
				XmNpacking, XmPACK_COLUMN,
				XmNnumColumns, 1, NULL );

	// use the supplied class instance name for the label widget text
	_label = XtCreateManagedWidget ( _name, xmLabelWidgetClass, _rc1, NULL, 0 );

	n = 0;
	XtSetArg ( args[n], XmNcolumns, _columns ); n++;
	XtSetArg ( args[n], XmNresizeWidth, TRUE ); n++;

	_text = XtCreateManagedWidget ( "text", xmTextFieldWidgetClass, _rc1, args, n );

	XtAddCallback ( _text, XmNactivateCallback, &LineEdit::enterCallback, ( XtPointer ) this );
	
	// now create another rowcol widget on the first; this time pack by rows
	_rc2 = XtVaCreateManagedWidget ( "RowColumn2", xmRowColumnWidgetClass, _rc1,
				XmNorientation, XmVERTICAL,
				XmNpacking, XmPACK_COLUMN,
				XmNnumColumns, 4, NULL );
				
	_ok = XtCreateManagedWidget ( "OK", xmPushButtonWidgetClass, _rc2, NULL, 0 );
	XtAddCallback ( _ok, XmNactivateCallback, &LineEdit::okCallback, ( XtPointer ) this );

	_cancel = XtCreateManagedWidget ( "Cancel", xmPushButtonWidgetClass, _rc2, NULL, 0 );
	XtAddCallback ( _cancel, XmNactivateCallback, &LineEdit::cancelCallback, ( XtPointer ) this );

	if ( _helpText[0] != 0 )	// if no help, don't need these buttons
	{
		_clear = XtCreateManagedWidget ( "Clear", xmPushButtonWidgetClass, _rc2, NULL, 0 );
		XtAddCallback ( _clear, XmNactivateCallback, &LineEdit::clearCallback, ( XtPointer ) this );

		_help = XtCreateManagedWidget ( "Help", xmPushButtonWidgetClass, _rc2, NULL, 0 );
		XtAddCallback ( _help, XmNactivateCallback, &LineEdit::helpCallback, ( XtPointer ) this );
	}
	
	// cater for WM 'X' pressed
	WM_DELETE_WINDOW = XmInternAtom ( XtDisplay ( _popup ), "WM_DELETE_WINDOW", False );
	XmAddWMProtocols ( _popup, &WM_DELETE_WINDOW, 1 );
	XmAddWMProtocolCallback ( _popup, WM_DELETE_WINDOW, clientMessageProc, ( XtPointer ) this );

	XtPopup ( _popup, ( XtGrabKind ) XtGrabNone );
	XtManageChild ( _popup );
}


void LineEdit::clientMessageProc ( Widget w, XtPointer clientData, XtPointer cbs)
{
	LineEdit *obj = ( LineEdit * ) clientData;
	obj->clientMessage ( ( XmAnyCallbackStruct * ) cbs );
		XtDestroyWidget ( w );
}


void LineEdit::clientMessage ( XmAnyCallbackStruct *cbs )
{
	if ( cbs->reason != XmCR_CANCEL && cbs->reason != XmCR_OK )
		cancel();
}


void LineEdit::enterCallback ( Widget w, XtPointer clientData, XtPointer callData )
{
	char buf[256];

	LineEdit *obj = ( LineEdit * ) clientData;				
//	XmAnyCallbackStruct *cb = ( XmAnyCallbackStruct * ) callData;
// could use int cb->reason and XEvent *cb->event here			
	
	snprintf ( buf, sizeof ( buf ), "%s", XmTextFieldGetString ( w ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	obj->enter( buf );
	
	XtDestroyWidget ( w );
}


void LineEdit::enter ( char * text )
{
	if ( _callback )
		_callback ( _clientData, text );

	XtPopdown ( _popup );
}


void LineEdit::okCallback ( Widget w, XtPointer clientData, XtPointer callData )
{
	char buf[256];

	LineEdit *obj = ( LineEdit * ) clientData;

	if ( !obj->_presetText )
		snprintf ( buf, sizeof ( buf ), "%s", XmTextFieldGetString ( obj->_text ) );
	else
	{
		snprintf ( buf, sizeof ( buf ), "%s", obj->_setText );
		obj->_presetText = 0;
	}
	
	buf [ sizeof ( buf ) - 1 ] = 0;
	obj->enter( buf );	// ok mapped to 'enter'
	XtDestroyWidget ( w );
}


void LineEdit::clearCallback ( Widget w, XtPointer clientData, XtPointer callData )
{
	LineEdit *obj = ( LineEdit * ) clientData;
	obj->clear();
}


void LineEdit::clear()
{
	char buf[1];
	
	buf[0] = 0;
	XmTextFieldSetString ( _text, buf );
	_presetText = 1;
}


void LineEdit::helpCallback ( Widget w, XtPointer clientData, XtPointer callData )
{
	LineEdit *obj = ( LineEdit * ) clientData;

	obj->help( clientData );	// preset help text supplied on invocation
}


void LineEdit::help( void *clientData )
{
	char buf[256];

	LineEdit *obj = ( LineEdit * ) clientData;

	snprintf ( buf, sizeof ( buf ), "%s", obj->_helpText );
	buf [ sizeof ( buf ) - 1 ] = 0;

	XmTextFieldSetString ( _text, buf );
	_presetText = 1;
}


void LineEdit::cancelCallback ( Widget w, XtPointer clientData, XtPointer callData )
{
	LineEdit *obj = ( LineEdit * ) clientData;
	obj->cancel();
	XtDestroyWidget ( w );
}


void LineEdit::cancel()
{
    if ( _cancelCallback )
		_cancelCallback ( _clientData );
	
	XtPopdown ( _popup );
}


void LineEdit::setText ( char *text )
{
	char buf[256];
	
	snprintf ( buf, sizeof ( buf ), "%s", text );
	buf [ sizeof ( buf ) - 1 ] = 0;
	strncpy ( _setText, buf, sizeof ( _setText ) );	// copy in case we invoke clear or help
	_setText [ sizeof ( _setText ) - 1 ] = 0;
	XmTextFieldSetString ( _text, buf );
}


void LineEdit::setEditStatus ( Boolean status )
{
	XtVaSetValues ( _text, XmNeditable, status, NULL );
}


void LineEdit::setColumns ( int columns )
{
	_columns = columns;
}


void LineEdit::close()
{
	cancel();
	XtDestroyWidget ( _popup );
}
