/*
 Interface.h - originally 'UIComoponent.h'
 basic user interface component class
 defines protocol for derived classes
 adds support for widget destruction and resource management to the
 Foundation class

 Douglas A. Young, 'Object-oriented Programming with C++ and OSF/Motif'
 2nd ed., Prentice-Hall
*/


#ifndef INTERFACE_H
#define INTERFACE_H

#include <Xm/Xm.h>
#include "Foundation.h"

class Interface : public Foundation
{
	public:
		virtual ~Interface();	// destructor
		virtual void manage();	// manage widget subtree

		// access function
		virtual const char *const className() { return ( "Interface" ); }

	protected:
		Interface ( const char * );	// don't instantiate constr.
		void installDestroyHandler();	// hook for derived classes

		// called by widgetDestroyCallback() if base widget destroyed
		virtual void widgetDestroyed();

		// load component's default resources into database
		void setDefaultResources ( const Widget, const String * );

		// retrieve this class' resources from resource manager
		void getResources ( const XtResourceList, int );

	private:
		// interface between XmNdestroyCallback and this class
		static void widgetDestroyedCallback ( Widget, XtPointer, XtPointer );
};

#endif
