// WordCountWindow.cc

#include <Application.h>
#include <InfoDialogManager.h>
#include "WordCountWindow.h"
#include "CountWordsCmd.h"
#include <FileSelect.h>
#include <QuitCmd.h>
#include <MenuBar.h>
#include <CmdList.h>
#include <Xm/List.h>
#include <X11/cursorfont.h>


WordCountWindow::WordCountWindow ( const char *name ) : MenuWindow ( name )
{
	// empty
}


Widget WordCountWindow::createWorkArea ( Widget parent )
{
	_list = XmCreateScrolledList ( parent, "list", NULL, 0 );
	XtManageChild ( _list );
	return ( XtParent ( _list ) );
}


void WordCountWindow::createMenuPanes()
{
	//create command onjects for menu
	Cmd *quit = new QuitCmd ( "Quit" );
	Cmd *fileSelect = new FileSelect ( "fileSelect", TRUE,
					   &WordCountWindow::countWordsCallback,
					   ( void * ) this );
									   
	// create  list of commands and install in menu pane
	CmdList *cmdList = new CmdList ( "Application" );
	cmdList->add ( fileSelect );
	cmdList->add ( quit );
	_menuBar->addCommands ( cmdList );
}


void WordCountWindow::countWordsCallback ( void *clientData, char *filename )
{
	WordCountWindow *obj = ( WordCountWindow * ) clientData;
	obj->countWords ( filename );
}


void WordCountWindow::countWords ( char *filename )
{
	if ( !filename )
		return;
	
	// instantiate Cmd object to count words in given file
	CountWordsCmd *task = new CountWordsCmd ( "countWords", TRUE, filename );
	setBusyCursor();
	
	XtVaSetValues ( _list, XmNitems, NULL, XmNitemCount, 0, NULL );
	
	// execute command, providing function to be called on completion
	task->execute ( &WordCountWindow::taskFinishedCallback, ( void * ) this );
}


void WordCountWindow::taskFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	CountWordsCmd *cwObj = ( CountWordsCmd * ) cmd;
	WordCountWindow *obj = ( WordCountWindow * ) clientData;
	
	if ( interrupted )
		infoDialogManager->post ( "Interrupted!" );
	else
		obj->taskFinished ( cwObj );
		
	obj->setNormalCursor();
	delete cwObj;
}


void WordCountWindow::taskFinished ( CountWordsCmd *cwObj )
{
	int i;
	char buf[100];
	XmString *xmstrList;
	sprintf ( buf, "This file contains %d unique words.", cwObj->numWords() );
	infoDialogManager->post ( buf );
	xmstrList = new XmString[cwObj->numWords()];
	
	for ( i = 0; i < cwObj->numWords(); i++ )
	{
		char buf[BUFSIZE];
		
		sprintf ( buf, "%-5d %s", cwObj->getCount ( i ), cwObj->getWord ( i ) );
		xmstrList[i] = XmStringCreateSimple ( buf );
	}
	// display array of compound strings in list
	// Warning! - this can cause refused resize from XtMakeGeometryRequest
	XtVaSetValues ( _list, XmNitems, xmstrList, XmNitemCount,
					cwObj->numWords(), NULL );
	for ( i = 0; i < cwObj->numWords(); i++ )
		XmStringFree ( xmstrList[i] );	// XmList widget makes own copies, free these

	delete[] xmstrList;
}


void WordCountWindow::setBusyCursor()
{
	if ( XtIsRealized ( _w ) )
	{
		if ( !_busyCursor )
			_busyCursor = XCreateFontCursor ( XtDisplay ( _w ), XC_watch );
			
		// install cursor for window's top-level shell
		XDefineCursor ( XtDisplay ( _w ), XtWindow ( _w ),  _busyCursor );
	}
}


void WordCountWindow::setNormalCursor()
{
	if ( XtIsRealized ( _w ) )
	{
		if ( !_normalCursor )
			_normalCursor = XCreateFontCursor ( XtDisplay ( _w ), XC_left_ptr );
			
		// install left ptr cursor as normal cursor for window's top-level shell
		XDefineCursor ( XtDisplay ( _w ), XtWindow ( _w ), _normalCursor );
	}
}
